#!/usr/bin/env python3

# Copyright (C) 2021 VMware Inc.

# Simple module to convert cloud-init network config V1 to V2
# This satisfies most of our use cases at the moment
# Need to add support to handle all other configs as needed


import ipaddress


def parse_subnet_routes(cfg, conf):
    # place holder for routes inside subnets
    pass


def parse_subnets(v2cfg, cfg, conf, if_name):
    for i in conf:
        for k, v in i.items():
            if k == 'type':
                if v == 'dhcp' or v == 'dhcp4':
                    cfg[if_name]['dhcp4'] = True
                if v == 'dhcp6':
                    cfg[if_name]['dhcp6'] = True
                if v == 'static' or v == 'static6':
                    cfg[if_name]['dhcp4'] = False
                    cfg[if_name]['dhcp6'] = False
            elif k == 'address':
                cfg[if_name]['addresses'] = [v]
            elif k == 'netmask':
                pass
                # ip4 = ipaddress.IPv4Network((0, v))
                # TODO: need to make use of this
                # ip4prefixlen = ip4.prefixlen
            elif k == 'gateway':
                cfg[if_name]['gateway4'] = v
            elif k == 'dns_nameservers':
                try:
                    cfg[if_name]['nameservers'].update({'addresses': v})
                except KeyError:
                    cfg[if_name]['nameservers'] = {'addresses': v}
            elif k == 'dns_search':
                try:
                    cfg[if_name]['nameservers'].update({'search': v})
                except KeyError:
                    cfg[if_name]['nameservers'] = {'search': v}
            elif k == 'routes':
                parse_subnet_routes(cfg, v)

    return cfg


def parse_phy(v2cfg, conf, if_name):
    cfg = {if_name: {}}
    cfg[if_name]['set-name'] = if_name
    for k, v in conf.items():
        if k == 'mac_address':
            cfg[if_name]['match'] = {'macaddress': v}
        elif k == 'subnets':
            subnet_cfgs = parse_subnets(v2cfg, cfg, v, if_name)
            cfg[if_name].update(subnet_cfgs)
        elif k == 'mtu':
            cfg[if_name]['mtu'] = v

    return cfg


def parse_routes(v2cfg, conf, if_name):
    cfg = {}
    for k, v in conf.items():
        if k == 'destination':
            cfg['to'] = v
        elif k == 'gateway':
            cfg['via'] = v
        elif k == 'metric':
            cfg[k] = v

    return cfg


def parse_nameserver(v2cfg, conf, if_name):
    cfg = {}
    for k, v in conf.items():
        if k == 'search':
            cfg[k] = v
        elif k == 'address':
            cfg['addresses'] = v

    return cfg


def v1tov2(cfg):
    key = 'ethernets'
    v2cfg = {key: {}}

    for conf in cfg:
        if 'name' in conf:
            if_name = conf['name']
        for _, v in conf.items():
            if v == 'physical':
                ret_cfg = parse_phy(v2cfg, conf, if_name)
                try:
                    v2cfg[key].update(ret_cfg)
                except KeyError:
                    v2cfg[key] = ret_cfg
            elif v == 'route':
                ret_cfg = parse_routes(v2cfg, conf, if_name)
                try:
                    v2cfg[key][if_name][v + 's'].append(ret_cfg)
                except KeyError:
                    v2cfg[key][if_name][v + 's'] = [ret_cfg]
            elif v == 'nameserver':
                ret_cfg = parse_nameserver(v2cfg, conf, if_name)
                try:
                    v2cfg[key][if_name][v + 's'].update(ret_cfg)
                except KeyError:
                    v2cfg[key][if_name][v + 's'] = ret_cfg

    return v2cfg
